! main.m
!
! Program to compute the Stochastic Neoclassical Growth Model using  Value Function Iteration
!
! Jesus Fernandez-Villaverde, adapted by Francisco Barillas New York, February 7th, 2006
! Philadelphia, February 13th, 2003

program myopt

	use mod_types
	use mod_common_var
	use dfport
	
	implicit none

	real(dp), allocatable, dimension(:,:)  :: transition, transitioncdf, value, g_k, g_c, g_l, euler_errors
	real(dp), allocatable, dimension(:)  :: y, grid_k

	integer :: index_k, index_z, length_grid_k

	real(dp) :: phi, omega, big_phi, k_ss, l_ss, c_ss, r_ss, y_ss
	real(dp) :: cover, cover_tauchen
	
	real (4) :: elapt, ta(2) !This is used in order to time the main block of the program.
	real(dp) :: t0,t1,t2,delta_t
	
	!Begins timing the main block.
	t0=rtc()
	t1=rtc()

	print *, ' '
	print *, 'Beginning of the Program Value Function Iteration'
	print *, ' '
	
	!----------------------------------------------------------------
	! 1. Computation of the Steady State
	!----------------------------------------------------------------

	phi     = (1/alpha*(1/beta-1+delta))**(1/(1-alpha))
	omega   = (phi**(1-alpha)-delta)
	big_phi = theta/(1-theta)*(1-alpha)*phi**(-alpha) 
  
	k_ss = big_phi/(omega+phi*big_phi)
	l_ss = phi*k_ss
	c_ss = omega*k_ss
	r_ss = alpha*(k_ss**(alpha-1))*(l_ss**(1-alpha))
	y_ss = k_ss**alpha*(l_ss**(1-alpha))

	print *, 'Steady State values'
	print *, 'Output: ', y_ss, 'Capital: ', k_ss, 'Labor: ', l_ss

	allocate(transition(n_tauchen,n_tauchen))
	allocate(transitioncdf(n_tauchen,n_tauchen))
	allocate(y(n_tauchen))

	!----------------------------------------------------------------
	! 2. Tauchen Points
	!----------------------------------------------------------------

	cover_tauchen = 3.d0  
	call sub_tauchen(y,transition,transitioncdf,rho,sigma,n_tauchen,cover_tauchen)

	print *, ' '
	print *, 'For the stochastic shock, Tauchen`s approximation method with', n_tauchen , 'points will be used.'
	print *, ' '

	!----------------------------------------------------------------
	! 3. Main Iteration
	!----------------------------------------------------------------
	
	print *, 'Beginning the iteration of the value function with an initial linear guess '
	print *, 'and the sup norm with a tolerance of ', toler
	print *, ' '

	cover = 0.25d0         ! Coverage of the grid (+- of steady state k)
	length_grid_k = 1000
	
	allocate(grid_k(length_grid_k))
	allocate(value(length_grid_k,n_tauchen))
	allocate(g_l(length_grid_k,n_tauchen))
	allocate(g_k(length_grid_k,n_tauchen))
	allocate(g_c(length_grid_k,n_tauchen))

	call sub_grid_generation(cover,length_grid_k,k_ss,grid_k)

	value = (((c_ss**theta)*((1-l_ss)**theta))**(1-tau))/(1-tau)
	call sub_value(value,g_k,g_c,g_l,grid_k,length_grid_k,y,transition)

	t2=rtc()
	delta_t=t2+t0-2.0d0*t1
	print *, ' '
	print *, 'Elapsed time in the main program =', delta_t, 'sec'
	print *, ' '

	! Computes euler equation errors
	allocate(euler_errors(length_grid_k,n_tauchen))
	call sub_eulerequationerror(euler_errors,value,g_k,g_c,g_l,grid_k,length_grid_k,y,transition)

	! Computes the average of the euler equation errors along the simulated path.
	call sub_simul(euler_errors,g_k,grid_k,length_grid_k,transition,transitioncdf,k_ss)

	!----------------------------------------------------------------
	! 4. Output
	!----------------------------------------------------------------
	
	open(unit=11,	file='value.txt', status = 'replace')
	write (11, '(41f20.10)') (value(i1, :), i1 = 1,length_grid_k)
	close(11)

	open(unit=12,	file='g_l.txt', status = 'replace')
	write (12, '(41f14.7)') (g_l(i1, :), i1 = 1,length_grid_k)
	close(12)
	
	open(unit=13,	file='g_k.txt', status = 'replace')
	write (13, '(41f14.7)') (g_k(i1, :), i1 = 1,length_grid_k)
	close(13)

	open(unit=14,	file='g_c.txt', status = 'replace')
	write (14, '(41f14.7)') (g_c(i1, :), i1 = 1,length_grid_k)
	close(14)
		
	elapt = etime(ta)
	write (*,*), ' '
	write (*,'(a17, x, f8.2, x, a20)'), ' Program has used', elapt, 'seconds of CPU time,'
	write (*,'(a5, x, f8.2, x, a24, x, f5.2, x, a23)'), ' with', ta(1), 'seconds of user time and', ta(2), 'seconds of system time.'

	print *, ' '
	print *, 'End of the program'
	print *, ' '

end program myopt